#! /usr/bin/env python2
###############################################################################
#
# simulavr - A simulator for the Atmel AVR family of microcontrollers.
# Copyright (C) 2001, 2002  Theodore A. Roth
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################
#
# $Id$
#

"""Test the BRBS opcode.

The following opcodes are pseudonyms for BRBS:
  BREQ, BRCS, BRLO, BRMI, BRLT, BRHS, BRTS, BRVS, BRIE
"""

import array, struct
import base_test
from registers import Reg

class BRBS_TestFail(base_test.TestFail): pass

class base_BRBS(base_test.opcode_test):
	"""Generic test case for testing BRBS opcode.

	Branch if sreg bit set.
	opcode is '1111 01kk kkkk ksss' where k is pc offset and s is sreg bit.

	No registers except for PC should have changed.
	"""
	# FIXME: Offsets can wrap around ends of flash. Need to mask expect value
	# with size of flash. Once done, need to test it.
	k = 20
	def setup(self):
		opcode = array.array( 'B', struct.pack('<H', 0xf000 | self.bit | ((self.k&0x7f)<<3)) )

		self.regs = self.target.read_regs()
		self.target.write_flash(self.regs[Reg.PC], len(opcode), opcode)

		# set the sreg bit we are interrested in
		self.regs[Reg.SREG] = self.val << self.bit
		self.target.write_reg(Reg.SREG, self.regs[Reg.SREG])

	def analyze_results(self):
		new_regs = self.target.read_regs()

		expect = self.regs[Reg.PC] + 2
		if self.val == 1:
			expect += (self.k * 2)

		if expect != new_regs[Reg.PC]:
			self.fail('PC not incremented: expect=%x, got=%x' % (expect, new_regs[Reg.PC]))

		# compare all regs except PC
		for i in range(Reg.PC):
			old = self.regs[i]
			new = new_regs[i]
			if old != new:
				self.fail('Register r%d changed: old=%x, new=%x' % (i, old, new))

#
# Template code for test case.
# The fail method will raise a test specific exception.
#
template = """
class BRBS_bit%d_is_%d_TestFail(BRBS_TestFail): pass

class test_BRBS_bit%d_is_%d(base_BRBS):
	bit = %d
	val = %d
	def fail(self,s):
		raise BRBS_bit%d_is_%d_TestFail, s
"""

#
# automagically generate the test_BRBS_bitN_is_[01] class definitions
#
code = ''
for b in range(8):
	for v in range(2):
		code += template % (b,v,b,v,b,v,b,v)
exec code
