#! /usr/bin/env python2
###############################################################################
#
# simulavr - A simulator for the Atmel AVR family of microcontrollers.
# Copyright (C) 2001, 2002  Theodore A. Roth
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################
#
# $Id$
#

"""Test the NOP opcode.
"""

import array
import base_test
from registers import Reg

class NOP_TestFail(base_test.TestFail): pass

class test_NOP(base_test.opcode_test):
	"""Test the NOP opcode.

	A NOP should only change the PC (PC <- PC + 1). If any other registers
	change, test fails.

	PC should be incremented by two bytes (1 word). Data sheet uses 1 word and
	gdb interface uses 2 bytes.
	"""

	def setup(self):
		# save the register values
		self.regs = self.target.read_regs()

		# write a NOP to current PC
		opcode = array.array( 'B', [0x0,0x0] ) # NOP 
		self.target.write_flash( self.regs[Reg.PC], len(opcode), opcode )

	def analyze_results(self):
		# get new register values
		regs = self.target.read_regs()

		# compare all regs except PC
		for i in range(Reg.PC):
			old = self.regs[i]
			new = regs[i]
			if old != new:
				raise TestFail_NOP, 'Register r%d changed: old=%02x, new=%02x' % (i, old, new)

		# expect pc to be += 2 since gdb uses bytes instead of words
		expect = self.regs[Reg.PC] + 2
		got    = regs[Reg.PC]
		if expect != got:
			raise NOP_TestFail, 'PC incorrectly incremented: expect=%x, got=%x' % (expect, got)

